/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * pro-common.c
 * Common functions for the Enttec pro
 * Copyright (C) 2005-2006 Simon Newton
 */

#include "pro-common.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <time.h>
#include <termios.h>
#include <errno.h>
#include <unistd.h>


/*
 * set the msg length
 */
static void set_msg_len(promsg *msg, int len) {
	msg->len = len & 0xFF ;
	msg->len_hi = (len & 0xFF00) >> 8 ;
}

/*
 * Send a generic msg
 */
static int send_msg(_pro_device *dev, promsg *msg) {
	uint8_t eom = PRO_EOM ;
	int len = (msg->len_hi << 8) + msg->len ;
	write(dev->fd, msg, len+4) ;
	write(dev->fd, &eom, sizeof(eom) ) ;

	return 0;
}


/*
 * Send a reprogram request
 *
 *
 */
int pro_send_reprogram(pro_device dev) {
	promsg msg ;
	_pro_device *d = (_pro_device*) dev ;	

	if(d == NULL)
		return -1 ;
	
	msg.som = PRO_SOM ;
	msg.label = PRO_ID_REPROG ;
	set_msg_len(&msg, 0);

	return send_msg(d, &msg) ;
}



/*
 * Send a flash page msg
 *
 *
 */
int pro_send_flsreq(pro_device dev, uint8_t *buf, int len) {
	int l = len < FLASH_PAGE ? len : FLASH_PAGE ;
	promsg msg ;
	_pro_device *d = (_pro_device*) dev ;	

	if(d == NULL)
		return -1 ;

	msg.som = PRO_SOM ;
	msg.label = PRO_ID_FLSREQ ;
	set_msg_len(&msg, l);
	memcpy(&msg.pm_flsreq.page, buf, l) ;

	return send_msg(d, &msg) ;
}




/*
 * Send a dmx msg
 *
 *
 */
int pro_send_dmx(pro_device dev, uint8_t *buf, int len) {
	promsg msg ;
	_pro_device *d = (_pro_device*) dev ;	

	if ( d == NULL) 
		return -1 ;

	msg.som = PRO_SOM ;
	msg.label = PRO_ID_SDMX ;
	set_msg_len(&msg, len);
	memcpy(&msg.pm_dmx.dmx, buf, len) ;

	return send_msg(d, &msg) ;
}


/*
 * Send a rdm msg
 *
 *
 */
int pro_send_rdm(pro_device dev, uint8_t *buf, int len) {
	promsg msg ;
	_pro_device *d = (_pro_device*) dev ;	

	if( d == NULL) 
		return -1 ;

	
	msg.som = PRO_SOM ;
	msg.label = PRO_ID_RDM ;
	set_msg_len(&msg, len);
	memcpy(&msg.pm_rdm.dmx, buf, len) ;

	return send_msg(d, &msg) ;
}



/*
 * Send a param request
 *
 */
int pro_send_prmreq(pro_device dev, int usrsz) {
	promsg msg ;
	_pro_device *d = (_pro_device*) dev ;	

	if(d == NULL)
		return -1 ;

	msg.som = PRO_SOM ;
	msg.label = PRO_ID_PRMREQ ;
	msg.pm_prmreq.len = usrsz & 0xFF;
	msg.pm_prmreq.len_hi = (usrsz & 0xFF) >> 8;
	
	return send_msg(d, &msg) ;
}

/*
 * Send a mode msg
 *
 *
 */
int send_rcmode(pro_device dev, int mode) {
	uint8_t m = mode ? RCMODE_CHANGE : RCMODE_ALWAYS ;
	_pro_device *d = (_pro_device*) dev ;	
	
	promsg msg ;

	if(d == NULL)
		return -1 ;

	msg.som = PRO_SOM ;
	msg.label = PRO_ID_RCMODE ;
	set_msg_len(&msg, sizeof(pms_rcmode) );
	msg.pm_rcmode.mode = m ;

	return send_msg(d, &msg) ;
}



/*
 * Send a serial number request
 *
 */
int pro_send_snoreq(pro_device dev) {
	promsg msg ;

	if(dev == NULL)
		return -1 ;

	msg.som = PRO_SOM ;
	msg.label = PRO_ID_SNOREQ ;
	set_msg_len(&msg, sizeof(pms_snoreq) );
	
	return send_msg(dev, &msg) ;
}



/*

int handle_cos(pms_cos *cos, int len) {
	int chn_st = cos->start *8;
	int i, offset ;
	
	printf(" got dmx cos\n");
	printf(" start %hhx\n", cos->start) ;
	printf(" %hhx  %hhx %hhx %hhx %hhx\n", cos->changed[0], cos->changed[1], cos->changed[2], cos->changed[3], cos->changed[4]) ;

	offset = 0 ;
	for(i = 0; i< 40; i++) {

		if( cos->changed[i/8] & (1 << (i%8)) ) {
			printf("index %i changed to %hhx\n", chn_st + i,  cos->data[offset]  ) ;
		//	dmx_buffer[chn_st + i] = cos->data[offset] ;
			offset++ ;
		}
	}
	

	return 0 ;

}
*/



/*
 * We have at least one byte to read here
 *
 */
int do_recv(_pro_device *dev) {
	uint8_t byte, label ;
	int cnt, unread, plen, bytes_read;
	pmu buf ;
	int fd = dev->fd ;
	
	while(byte != 0x7e) {
		cnt = read(fd,&byte,1) ;

		if(cnt != 1) {
//			printf("read to much %i\n", cnt) ;
			return -1 ;
		}

	}

	// try to read the label
	cnt = read(fd,&label,1) ;
	
	if(cnt != 1) {
//		printf("read to much %i\n", cnt);
		return 1 ;
	}

	cnt = read(fd, &byte,1);
	if (cnt != 1) { 
//	    printf("read to much %i\n", cnt);
    	return 1;
	}
	plen = byte ;
	
	cnt = read(fd, &byte,1);
	if (cnt != 1) { 
//	    printf("read to much %i\n", cnt);
    	return 1;
	}
	plen += byte<<8;

	for(bytes_read = 0; bytes_read < plen;) {
		bytes_read += read(fd, (uint8_t*) &buf +bytes_read, plen-bytes_read) ;
	}

	// check here

	// check this is a valid frame with an end byte
	cnt = read(fd, &byte,1);
	if (cnt != 1) { 
//	    printf("read to much %i\n", cnt);
    	return 1;
	}

	if(byte == 0xe7) {
		// looks ok
	
		switch(label) {
			case PRO_ID_FLSREP:
				if ( dev->flsrep_h != NULL) {
					dev->flsrep_h((pro_device) dev, &buf, cnt) ;
				}
				break ;
			case PRO_ID_RDMX:
			//	handle_dmx( &buf.pmu_rdmx, cnt) ;
				break;
			case PRO_ID_PRMREP:
				if ( dev->prmrep_h != NULL) {
					dev->prmrep_h((pro_device) dev, &buf, cnt) ;
				}
				break;
			case PRO_ID_COS:
		//		handle_cos( &buf.pmu_cos, cnt) ;
				break;
			case PRO_ID_SNOREP:
		//		handle_snorep( &buf.pmu_snorep, cnt) ;
				break;
			default :
				printf("Not sure what msg this is\n") ;
		}
	}

	ioctl(fd, FIONREAD, &unread) ;

	return 0 ;
}


/*
 * @param dev 	the pro device
 * @timeout		number of seconds to block for a reply.
 *
 */
int pro_recv(pro_device dev, int timeout) {
	_pro_device *d = (_pro_device*) dev ;	
		
	fd_set rset ;
	struct timeval tv ;
	int unread ;
	FD_ZERO(&rset) ;
 	FD_SET(d->fd, &rset) ;
	
	if(dev == NULL) 
		return -1 ;


	tv.tv_usec = 0 ;
	tv.tv_sec = timeout ;

	switch ( select( d->fd+1, &rset, NULL, NULL, &tv ) ) {
		case 0:
			// timeout
			return 1;
			break ;
		case -1:
			if( errno != EINTR) {
				printf("%s : select error", __FUNCTION__) ;
				return 1;
			}
			return 0;			
			break ;
		default:
			ioctl(d->fd, FIONREAD, &unread) ;

			while(unread != 0) {
				do_recv(d) ;
				ioctl(d->fd, FIONREAD, &unread) ;
			}
			break;
	}

	return 0 ;
}


/*
 * return the fd
 *
 */
int pro_getfd(pro_device dev) {
	_pro_device *d = (_pro_device*) dev ;	

	if(d == NULL)
		return -1 ;


	return d->fd; 

}


/*
 * Connect to the specified device
 *
 * @return 	fd or -1 on error
 */
pro_device pro_connect(char *dev_path, int verbose) {
	struct termios newtio ;
	_pro_device *dev ;
	int fd = open(dev_path, O_RDWR | O_NONBLOCK ) ;

	if(fd == -1) {
		if(verbose)
			printf("Could not open %s : %s\n", dev_path , strerror(errno) ) ;
		return NULL ;
	}

	bzero(&newtio, sizeof(newtio)); /* clear struct for new port settings */
	tcsetattr(fd,TCSANOW,&newtio);

	dev = (_pro_device*) malloc(sizeof(_pro_device)) ;

	if(dev == NULL) {
		if(verbose)
			printf("Could not malloc\n") ;
		close(fd) ;
		return NULL;
	}

	dev->prmrep_h = NULL ;
	dev->flsrep_h = NULL ;
	dev->verbose = verbose ;
	
	dev->fd = fd;
	return (pro_device*) dev ;
}


int pro_disconnect(pro_device dev) {
	_pro_device *d = (_pro_device*) dev ;	

	if(d == NULL)
		return -1 ;
		
	close(d->fd) ;

	free(dev) ;
	return 0;
}

int pro_set_handler(pro_device dev, pro_handler hdl ,  int (*handler)(pro_device dev, pmu *msg, int len) ) {
	_pro_device *d = (_pro_device*) dev ;	
	if(d == NULL)
		return -1 ;

	switch (hdl) {
		case PRO_PRMREP_H:
				d->prmrep_h = handler ;
				break ;
		case PRO_FLSREP_H:
				d->flsrep_h = handler ;
		default:

				break; 
	}
		


	return 0;
}
