/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * pro-detect.c
 * Copyright (C) 2005-2006 Simon Newton
 */

#include "pro-common.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <time.h>

#define DEFAULT_DEVICE "/dev/ttyUSB0" 


typedef struct {
	int verbose;
	int help;
	char *device;
} options;

// state of reply msg
int got_reply = 2 ;

// our command line options
options opts ;


/*
 * Called when we get a param reply
 */
int handle_param_reply(pro_device dev, pmu *msg, int len) {

	int frmvr = msg->pmu_prmrep.firmv + (msg->pmu_prmrep.firmv_hi <<8) ;
	
	if(opts.verbose) {
		printf("Firmware is %i\n", frmvr ) ;
		printf("Break time %hhx\n", msg->pmu_prmrep.brtm) ;
		printf("MAB time%hhx\n", msg->pmu_prmrep.mabtm) ;
		printf("Rate %hhx\n", msg->pmu_prmrep.rate) ;
	}

	got_reply = 1 ;
	return 0 ;
}

/*
 * Init options
 */
void init_options(options *opts) {
	opts->verbose =  0 ;
	opts->help =  0 ;
	opts->device = DEFAULT_DEVICE ;
	
}

/*
 * parse our cmd line options
 *
 */
int parse_options(int argc, char *argv[], options *opts) {
	static struct option long_options[] = {
			{"verbose", 	no_argument, 		0, 'v'},
			{"help", 		no_argument, 		0, 'h'},
			{0, 0, 0, 0}
		};

	int c ;
	int option_index = 0;
	
	while (1) {
     
		c = getopt_long(argc, argv, "vh", long_options, &option_index);
		
		if (c == -1)
			break;

		switch (c) {
			case 0:
				break;
			case 'v':
				opts->verbose = 1 ;
				break ;
			case 'h':
				opts->help = 1 ;
				break;
			case '?':
				break;
     
			default:
				;
		}
	}

	if (optind < argc) {
		opts->device = argv[optind++] ;
	}

	return 0;
}


/*
 * Display the help message
 */
void display_help_and_exit(char *argv[]) {

	printf(
"Usage: %s [device] \n"
"\n"
"Detect and report settings for a Enttec USB Pro device.\n"
"\n"
"  -h, --help        Display this help message and exit.\n"
"  -v, --verbose     Display the settings for the widget.\n"
"\n"
"Return values:\n"
" 0 - no device found\n"
" 1 - device found and working\n"
" 2 - device requires firmware\n",
		argv[0]) ;
	
	exit(0);
}


/*
 * Main
 *
 * @returns	0 on no device
 *	 		1 on ok
 * 			2 when the device requires flashing
 *
 */
int main(int argc , char *argv[] ) {
	pro_device *dev ;
	struct timespec tv ;

	tv.tv_sec = 0 ;
	tv.tv_nsec = 1000000 ;

	init_options(&opts);
	parse_options(argc, argv, &opts) ;

	if(opts.help)
		display_help_and_exit(argv) ;

	dev = pro_connect(opts.device, opts.verbose) ;

	if( dev == NULL) {
		return 0 ;
	}

	pro_set_handler(dev, PRO_PRMREP_H , handle_param_reply) ;

	// this seems to work well if we're the first program to open the device
	pro_send_prmreq(dev,0) ;

	nanosleep(&tv, NULL) ;

	pro_send_prmreq(dev,0) ;

	pro_recv(dev,1 );
	
	pro_disconnect(dev) ;
	return got_reply ;
}
